#include <stdio.h>      // must be included before idl_export.h
#include "idl_export.h" // brings in all the IDL definitions

static IDL_MSG_DEF msg_arr[] = {  
#define M_MG_WRONG_TYPE       0  
  {  "M_MG_WRONG_TYPE",   "%NThe input array must be of type float." },  
#define M_MG_VARINFO         -1    
  {  "M_MG_VARINFO",      "%N%d element(s), typecode=%d\nflags=%s" },  
};
static IDL_MSG_BLOCK msg_block; 

static IDL_VPTR IDL_mg_float_total(int argc, IDL_VPTR argv[]) {
  float *arr_data, total = 0.0;
  int i;
  IDL_VPTR result;
  IDL_VPTR arr = argv[0];
  
  // make sure we have a float array
  IDL_ENSURE_SIMPLE(arr);
  IDL_ENSURE_ARRAY(arr);
  if (arr->type != IDL_TYP_FLOAT) {
    IDL_MessageFromBlock(msg_block, M_MG_WRONG_TYPE, IDL_MSG_LONGJMP);
  }
  
  // do the calculation
  arr_data = (float *) arr->value.arr->data;
  for (i = 0; i < arr->value.arr->n_elts; i++) {
    total += (*arr_data++);
  }
  
  // put the result in an IDL variable
  result = IDL_Gettmp();
  result->type = IDL_TYP_FLOAT;
  result->value.f = total; 
  
  return(result);
}


static void IDL_mg_varinfo(int argc, IDL_VPTR argv[]) {
  char flags[128];
  IDL_VPTR arr = argv[0];

  sprintf(flags, "%s%s%s%s%s%s%s", 
          arr->flags & IDL_V_CONST ? "IDL_V_CONST " : "",
          arr->flags & IDL_V_TEMP ? "IDL_V_TEMP " : "",
          arr->flags & IDL_V_ARR ? "IDL_V_ARR " : "",
          arr->flags & IDL_V_FILE ? "IDL_V_FILE " : "",
          arr->flags & IDL_V_DYNAMIC ? "IDL_V_DYNAMIC " : "",
          arr->flags & IDL_V_STRUCT ? "IDL_V_STRUCT " : "",
          arr->flags & IDL_V_NOT_SCALAR ? "IDL_V_NOT_SCALAR " : "");
  
  if (arr->flags & IDL_V_ARR) {
    IDL_MessageFromBlock(msg_block, M_MG_VARINFO, IDL_MSG_RET, 
                         arr->value.arr->n_elts, arr->type, flags);
  } else {
    IDL_MessageFromBlock(msg_block, M_MG_VARINFO, IDL_MSG_RET, 
                         1, arr->type, flags);
  }
}


static void IDL_mg_print(int argc, IDL_VPTR argv[], char *argk) {
  int nargs;
  char *format, *cformat;
  IDL_VPTR origFormat, vcformat;
  
  typedef struct {
    IDL_KW_RESULT_FIRST_FIELD;
    IDL_VPTR format;
    int format_present;
  } KW_RESULT;
  KW_RESULT kw;
  
  static IDL_KW_PAR kw_pars[] = {
    { "FORMAT", IDL_TYP_STRING, 1, IDL_KW_VIN,
      IDL_KW_OFFSETOF(format_present), IDL_KW_OFFSETOF(format) },
    { NULL }
  };
  

  nargs = IDL_KWProcessByOffset(argc, argv, argk, kw_pars, NULL, 1, &kw);
  
  if (kw.format_present) {
    origFormat = argv[argc - 1];
    format = IDL_VarGetString(origFormat);
    cformat = (char *) calloc(strlen(format) + 5 + 1, sizeof(char));
    sprintf(cformat, "(%%\"%s\")", format);
    vcformat = IDL_StrToSTRING(cformat);
    argv[argc - 1] = vcformat;
  }
  
  IDL_Print(argc, argv, argk);
  
  if (kw.format_present) {
    argv[argc - 1] = origFormat;
    IDL_Deltmp(vcformat);
    free(cformat);
  }
  
  IDL_KW_FREE; 
}


int IDL_Load(void) {  
  /*
     These tables contain information on the functions and procedures
     that make up the mg_example_dlm DLM. The information contained in these
     tables must be identical to that contained in mg_example_dlm.dlm.
   */
  static IDL_SYSFUN_DEF2 function_addr[] = {
    { IDL_mg_float_total,         "MG_FLOAT_TOTAL",   1, 1, 0, 0 },
  };

  static IDL_SYSFUN_DEF2 procedure_addr[] = {
    { (IDL_SYSRTN_GENERIC) IDL_mg_varinfo,  "MG_VARINFO",  1, 1, 0, 0 },
    { (IDL_SYSRTN_GENERIC) IDL_mg_print,    "MG_PRINT",    0, IDL_MAXPARAMS, IDL_SYSFUN_DEF_F_KEYWORDS, 0 },   
  };

  if (!(msg_block = IDL_MessageDefineBlock("MG_Example_DLM", 
                                           IDL_CARRAY_ELTS(msg_arr),  
                                           msg_arr))) return IDL_FALSE;  
                                             
  // Register our routines.
  return IDL_SysRtnAdd(procedure_addr, FALSE, IDL_CARRAY_ELTS(procedure_addr))
      && IDL_SysRtnAdd(function_addr,  TRUE,  IDL_CARRAY_ELTS(function_addr));  
}